@@ -18,6 +18,7 @@ from . import fp23d
 from . import export3D 
 from .export import max_vertices
 from . import helpers as h
+from . import post_processing
 
 
 def main():
@@ -60,10 +61,10 @@ def main():
                         help='If scaling parameter is not used this is used to set half the size of the 3D structure')
     parser.add_argument('--max-vertices', type=int, default=max_vertices,
                         help='This will set the maximum nuber of vertices in the output 3D file.')
-    parser.add_argument('--threed-align', action='store_true',
-                        help='Attempt to do an aligment of the third dimension for multiple reconstructions')
+    parser.add_argument('--temporal-alignment', action='store_true',
+                        help='Attempt to do an aligment of the third dimension for multiple reconstructions over time. Here, all input images must have the same shape. The first and second images can not be aligned for the current implementation.')
     args = parser.parse_args()
-    method_args = ['negative_theta', 'same']
+    method_args = ['negative_theta']
     d_args = {key: vars(args)[key] for key in method_args}
 
     output_image_type = np.uint8
@@ -101,10 +102,8 @@ def main():
         global_mask = segmented == 0
         global_roi = Roi.find_from_mask(global_mask)
 
-    max_roi = Roi()
-
     ###### Demodulation of files ######
-    reconstructed = []
+    reconstructions = []
     for input_file in tqdm(to_reconstruct, disable=len(to_reconstruct) == 1):
         input_dir, input_filename = osp.split(input_file)
         input_filename_base, _ = osp.splitext(input_filename)
@@ -141,8 +140,6 @@ def main():
             roi = global_roi
         else:
             roi = Roi.full(signal.shape)
-        if args.same:
-            roi = Roi.full(signal.shape)
 
         texture_file = osp.join(input_dir, input_filename_base + '_texture.png')
         texture = None
@@ -152,46 +149,56 @@ def main():
 
 
         grid3d = fp23d(signal, calibration, **d_args)
-        reconstructed += [{'filename': input_file, 'name': '{}{}'.format(args.prefix, input_filename_base), 'grid': grid3d, 'texture': texture, 'calibration': calibration, 'roi': roi}]
-
-        if max_roi.empty() or list(signal.shape) == max_roi.full_shape():
-            max_roi.maximize(roi)
-
-        # plt.subplot(121)
-        # plt.imshow(signal)
-        # plt.subplot(122)
-        # plt.imshow(threeD)
-        # plt.show()
+        reconstructions.append({'filename': input_file, 'name': '{}{}'.format(args.prefix, input_filename_base), 'grid': grid3d, 'texture': texture, 'calibration': calibration, 'roi': roi})
 
 
     ###### Post processing of 3D files ######
-    if len(reconstructed) == 0:
-        print("No files were reconstructed")
+    if len(reconstructions) == 0:
+        print("No files were reconstructions")
         exit()
     if len(to_reconstruct) > 1:
         print('Working on 3D files')
 
-    if args.print_roi:
-        max_roi.write(osp.join(input_dir, args.roi_file))
-    roi = reconstructed[0]['roi'].copy()
-    roi.rebase(max_roi)
+    if not args.same:
+        # Finding minimum roi that fits all data
+        max_roi = reconstructions[0]["roi"].copy()
+        for reconstruction in reconstructions[1:]:
+            if not list(signal.shape) == max_roi.full_shape():
+                # If the original images have different shapes it makes no sense to use similar rois
+                max_roi = None
+                break
+            max_roi.maximize(roi)
 
-    base_grid = roi.unapply(reconstructed[0]['grid'])
-    for reconstruction in reconstructed:  # (input_file, grid3d, texture, calibration, roi) in tqdm(reconstructed):
+        if not max_roi is None and args.print_roi:
+            max_roi.write(osp.join(input_dir, args.roi_file))
+
+        # Upscaling the images again to the same shape or different
+        if not max_roi is None:
+            for reconstruction in reconstructions[1:]:
+                reconstruction['grid'] = max_roi.apply(reconstruction['grid'])
+
+    if args.temporal_alignment:
+        post_processing.temporal_alignment(reconstructions)
+
+    # Deducing what output file to use
+    if args.output is None:
+        input_dir = osp.dirname(reconstructions[0]['filename'])
+        output_file = osp.join(input_dir, '{}.{}'.format(reconstructions[0]['name'], args.ext))
+    elif osp.isdir(args.output):
+        output_file = osp.join(args.output, '{}.{}'.format(reconstructions[0]['name'], args.ext))
+    else:
+        output_file = args.output
+
+    #### Export to 3D file #######
+    export3D(output_file, reconstructions, args.out_3D_size, args.max_vertices, args.prefix)
+    ##############################
+
+    # Possible extra work
+    for reconstruction in reconstructions:
         input_dir = osp.dirname(reconstruction['filename'])
         output_dir = input_dir if args.output is None else osp.dirname(args.output)
-        # input_filename_base = osp.splitext(reconstruction['name'])[0]
 
-        if args.threed_align and roi.full_shape() == max_roi.full_shape():
-            roi.rebase(max_roi)
-            reconstruction['grid'] = roi.unapply(reconstruction['grid'])
-            diff = reconstruction['grid'][2] - base_grid[2]
-            l = np.median(np.ma.compressed(diff))
-            reconstruction['grid'] -= l
-            base_grid = reconstruction['grid']
-
-        grid3d = reconstruction['grid'].copy()
-        np.ma.set_fill_value(grid3d, 0)
+        grid3d = reconstruction['grid']
         if np.ma.isMaskedArray(grid3d):
             grid3d.data[grid3d.mask] = np.nan
 
@@ -203,21 +210,12 @@ def main():
             np.savetxt(osp.join(output_dir, '{}.csv'.format(reconstruction['name'])), ascii_array, fmt='%.3e')
 
         if args.print_image:
-            threeD = h.image_scaled(np.ma.filled(grid3d[2]), (0, np.iinfo(output_image_type).max))
-            threeD[grid3d.mask[2]] = 0
+            # Only printing the image of the third dimension values
+            threeD = h.image_scaled(grid3d[2], (0, np.iinfo(output_image_type).max))
+            threeD[np.isnan(threeD)] = 0
             output_file = osp.join(output_dir, '{}.png'.format(reconstruction['name']))
-            io.imsave(output_file, threeD.astype(output_image_type))
+            io.imsave(output_file, threeD.astype(output_image_type), check_contrast=False)
 
-    if args.output is None:
-        input_dir = osp.dirname(reconstructed[0]['filename'])
-        output_file = osp.join(input_dir, '{}.{}'.format(reconstructed[0]['name'], args.ext))
-    elif osp.isdir(args.output):
-        output_file = osp.join(args.output, '{}.{}'.format(reconstructed[0]['name'], args.ext))
-    else:
-        output_file = args.output
-
-    # Export 3D to a single file
-    export3D(output_file, reconstructed, args.out_3D_size, args.max_vertices, args.prefix)
 
 if __name__ == '__main__':
     main()
